function [ im_new, ssim_atgsp, coef_used ] = ATGSP( original_image, at_image, pointlist, quality, smooth_quality)
%ATGSP Performs the ATGSP post-processing algorithm
%
%   Input parameters:
%       original_image : Original image to be compressed.
%       at_image       : Adaptive thinning reconstruction.
%       pointlist      : Significant pixels of AT reconstruction as .inside file.
%       quality        : Overall quality of reconstruction, must be between 1 and 100.
%       smooth_quality : (optional) Quality of smooth image blocks, must be between 0 and 1.
%   Output parameters: 
%       im_new         : Reconstructed image with ATGSP.
%       ssim_atgsp     : SSIM of reconstruction.
%       coef_used      : Coefficients used in reconstruction.
% 
% Example: [atgsp_im,atgsp_ssim,atgsp_coef] = ATGSP('images/lena256.pgm', 'images/lena256_61536.pgm','images/lena256_61536.inside',20);

%Import Original image
im = imread(original_image);
resolution = size(im);

%Construct triangle matrix
[im_tri, size_tri] = CreateTriangleMatrix(pointlist,resolution);

%AT reconstruction
im_at_ansatz = imread(at_image);

%Difference matrix between image and AT approximation.
im_dif = double(im)-double(im_at_ansatz);
%Approximation of im_dif to be computed
im_dif_appr = zeros(resolution);

%Set quality factor
if quality > 50
    qF = 60/(110-quality);
else
    qF = (quality+25)/75;
end

%Set parameters based on quality
MAX_DATA_PER_TRIANGLE = floor(max(5,10*qF));               
CUTOFF_SIZE_COMPLEX = floor(max([0.00075*resolution(1)*resolution(2)/qF;...
                        0.0005*resolution(1)*resolution(2);20]));
CUTOFF_TEXTURED_PERCENTAGE = quality/100;
CUTOFF_SIZE_SMALL = 5;
CUTOFF_ST_VALUE = 35;
CUTOFF_COHERENCE = 0.1;

if exist('smooth_coefficients','var')
    SMOOTH_PERCENTAGE = smooth_quality;
else
    SMOOTH_PERCENTAGE = 0.05;
end


%Compute horizontal and vertical gradient matrix 
[Gx,Gy] = ComputeGradientMatrix(im_dif, im_tri, size(im_dif));

%Compute elements necessary for construction of the structure tensor
GxGy = Gx.*Gy;  %elementwise product of Gradients
Gx = Gx.^2;     %elementwise square of Gx
Gy = Gy.^2;     %elementwise square of Gy
% Array of directional vectors
directions = zeros(4,2);
for i = 1:4
   directions(i,:) = [cos((i-1)*pi/4) sin((i-1)*pi/4)];
end

%Initialisation
signal = cell(length(size_tri),1);
indices = signal;
coefficients = signal;
L = signal;
blocks_textured = zeros(length(size_tri),1);
blocks_smooth = blocks_textured;
blocks_complex_graph = blocks_textured;
classification = strings(length(size_tri),1);
coef_text = 0;


%Classification of image blocks based on the structure tensor
for i = 1:length(size_tri)
    if size_tri(i) > 0
        indices{i} = find(im_tri == i);
        
        %Create signal to be compressed
        signal{i} = im_dif(indices{i});
            
        %Classify triangle block using the structure tensor
        if size_tri(i) > CUTOFF_SIZE_SMALL
            %Compute classification
            classification(i) = BlockClassification(size_tri(i), Gx, Gy,...
                GxGy, indices{i}, CUTOFF_COHERENCE, CUTOFF_ST_VALUE, directions);
            if strcmp(classification(i), 'smooth') %smooth blocks
                blocks_smooth(i) = i;
            elseif strcmp(classification(i), 'complex') %complex blocks
                blocks_textured(i) = i;
                    if size_tri(i) > CUTOFF_SIZE_COMPLEX %blocks with complex graph
                        blocks_complex_graph(i) = i;
                    end
            else % blocks with dominant principal gradient
                blocks_textured(i) = i;
            end
        else %small blocks
            classification(i) = 'smooth';
            blocks_smooth(i) = i;
        end        
    end    
end

blocks_smooth = nonzeros(blocks_smooth);    
blocks_textured = nonzeros(blocks_textured);
blocks_complex_graph = nonzeros(blocks_complex_graph);


%Priority queue based on impact on SSIM for textured triangles
ssim_textured = zeros(length(blocks_textured),3);
for i = 1:length(blocks_textured)
    cur = blocks_textured(i); %Current block
    ssim_textured(i,1) = cur;
    ssim_textured(i,2) = LocalSSIM(double(im(indices{cur})),...
        double(im_at_ansatz(indices{cur})));
end
    
ssim_textured = sortrows(ssim_textured,2, 'descend');
extreme_coef = zeros(length(blocks_textured),2);


%Analysing textured blocks
for i = 1:floor(CUTOFF_TEXTURED_PERCENTAGE*length(blocks_textured))
    
    cur = ssim_textured(i,1); %Current block    
    
    %Create weight matrix depending on classification
    if strcmp(classification(cur), 'complex')
        if size_tri(cur) <= CUTOFF_SIZE_COMPLEX
            classification(cur) = 'smooth';
            W = CreateSmoothWeightMatrix(indices{cur},resolution,8);
        else
            continue %Complex matrices are created later with Delta
        end
    else
        W = CreateDominantWeightMatrix(indices{cur}, resolution,...
            classification(cur), signal{cur});
    end
    
    %Create graph structure
    L{cur} = gsp_graph(W);
    
    %Compute Fourier basis
    L{cur} = gsp_compute_fourier_basis(L{cur});

    %Computes GFT of signal
    coefficients{cur} = gsp_gft(L{cur},signal{cur});    
    
    extreme_coef(i,1) = min(coefficients{cur});
    extreme_coef(i,2) = max(coefficients{cur});    
end

%Quantization support
min_coef = floor(min(extreme_coef(:,1)));
max_coef = ceil(max(extreme_coef(:,2)));
rel_quant_support = 256/abs(max_coef-min_coef);
%Quantization step size
Delta = (max_coef - min_coef)/15;

%Construction and quantization of weight vectors of complex adaptive graphs
for i = 1:length(blocks_complex_graph)
    cur = blocks_complex_graph(i); %Current block
    
    [W, num_coef] = CreateComplexWeightMatrix(indices{cur}, resolution, signal{cur} ,Delta);
    coef_text = coef_text + num_coef;   %graph description cost
    
    %Create graph structure
    L{cur} = gsp_graph(W);

    %Compute Fourier basis
    L{cur} = gsp_compute_fourier_basis(L{cur});

    %Computes GFT of signal f
    coefficients{cur} = gsp_gft(L{cur},signal{cur});    
end


%Quantizing textured blocks
for i = 1:floor(CUTOFF_TEXTURED_PERCENTAGE*length(blocks_textured))
    
    cur = ssim_textured(i,1); %Current block   
    coef = coefficients{cur};

    %Select significant coefficients
    [~,index] = sort(abs(coef),'descend');    %sort data vector descending
    coefficients_new = zeros(length(signal{cur}),1);  %new data vector
    
    %Fill new vector with specified number of absolutely largest coefficients    
    for j = 1:min(MAX_DATA_PER_TRIANGLE,max(1,floor(length(signal{cur})*0.15)))        
       coefficients_new(index(j)) = round(coef(index(j))/Delta);   
       coef_text = coef_text + 1;   %coefficient cost
    end
    coef_text = coef_text + 1;    %overhead cost
       
    %Inverse GFT
    signal_new = gsp_igft(L{cur},coefficients_new*Delta);
        
    %Construct approximation of texture part
    im_dif_appr(indices{cur}) = signal_new;        
end


%Smooth triangles
ssim_smooth = zeros(length(blocks_smooth),2);


%Priority queue based on impact on SSIM for smooth triangles
for i = 1:length(blocks_smooth)
    cur = blocks_smooth(i); %Current block

    if size_tri(cur) >=CUTOFF_SIZE_SMALL
        ssim_smooth(cur,1) = cur;
        ssim_smooth(cur,2) = LocalSSIM(double(im(indices{cur})),...
            double(im_at_ansatz(indices{cur})));
    end
end

ssim_smooth = ssim_smooth(any(ssim_smooth,2),:);
ssim_smooth = sortrows(ssim_smooth,2, 'descend');

coef_smooth = 0;
smooth_index = 1;

%Analysing and quantizing smooth blocks
while coef_smooth < floor(coef_text * SMOOTH_PERCENTAGE) && smooth_index <= length(ssim_smooth)

    cur = ssim_smooth(smooth_index,1); %Current block
    smooth_index = smooth_index +1;

    %Create smooth weight matrix
    W = CreateSmoothWeightMatrix(indices{cur},resolution,8);

    %Create graph structure
    L{cur} = gsp_graph(W);  

    %Compute Fourier basis
    L{cur} = gsp_compute_fourier_basis(L{cur});

    %Computes GFT of signal f
    coefficients = gsp_gft(L{cur},signal{cur});    
    
    %Quantizes and thresholds coefficients
    coefficients = round(rel_quant_support*coefficients./...
        (ComputeQuantizationVector(L{cur}, size_tri(cur))));        
    coef_smooth = coef_smooth + nnz(coefficients) + sign(nnz(coefficients));         
    
    %Inverse GFT 
    signal_new = gsp_igft(L{cur},coefficients.*...
        (ComputeQuantizationVector(L{cur}, size_tri(cur))/rel_quant_support));
    
    %Add smooth part to approximation
    im_dif_appr(indices{cur}) = signal_new;        
end

%Reconstruct image
im_new = uint8(double(im_at_ansatz) + im_dif_appr);
%SSIM of reconstruction
ssim_atgsp = ssim(im_new,im);
%Estimation of coefficients used
coef_used = coef_text + coef_smooth;
end