function [ edge_weight_quantized, edge_signal ] = ComputeEdgeWeight(signal, incidence_matrix )
%ComputeEdgeWeight Computes edge weight for an adaptive graph
%
%   Input parameters:
%       signal                : Signal on image block.
%       incidence_matrix      : Incidence matrix of underlying graph.
%   Output parameters: 
%       edge_weight_quantized : Quantized optimal edge weight.
%       edge_signal           : Reconstructed edge weight vector.

%Differences over all edges
dif_edges = incidence_matrix * signal;

%Initialization
mean_edges = mean(abs(dif_edges));
edge_weight = 1;

%Classify edges and construct edge signal
while(edge_weight > 0.4)
    
    edge_signal = abs(dif_edges)-mean_edges;
    
    %Smooth edges
    smooth_edges = dif_edges(edge_signal <= 0);
    num_smooth_edges = length(smooth_edges);
    
    %Compute variance of smooth edges
    mean_smooth_edges = sum(smooth_edges)/num_smooth_edges;
    variance_smooth_edges = sum((smooth_edges-mean_smooth_edges).^2)/num_smooth_edges;
    
    %Transition edges 
    transition_edges = abs(dif_edges(edge_signal > 0) - mean_smooth_edges);    
    num_transition_edges = length(transition_edges); 
    
    %Compute variance of transition edges
    mean_transition_edges = sum(transition_edges)/num_transition_edges;
    variance_transition_edges = sum((transition_edges-mean_transition_edges).^2)/num_transition_edges;

    %Compute edge weight 
    edge_weight = variance_smooth_edges/(variance_transition_edges + variance_smooth_edges);
    
    %Reduce threshold for possible next iteration
    mean_edges = mean_edges - mean_edges/10;
end

%Constrain edge weight
if edge_weight < 0.05
    edge_weight = 0.05;
elseif edge_weight > 0.5
    edge_weight = 0.5;
elseif isnan(edge_weight)
    edge_weight = 0.2;
end

%Quantization of edge weight
edge_weight_quantized = (floor((edge_weight-0.025)/0.05)+1)*0.05;
end

