function [ W, num_coef] = CreateComplexWeightMatrix(indices, resolution, signal, Delta)
%CreateComplexWeightMatrix Creates weight matrix of a complex block
% 
%   Input parameters:
%       indices     : Indices of image block in the image.
%       resolution  : Resolution of original image.
%       signal      : Signal on image block.
%       Delta       : Quantization step size.
%   Output parameters:
%       W           : Quantized adapted weight matrix.
%       num_coef    : Coefficients used to reconstruct W.

%Construct incidence matrix
incidence_matrix = CreateIncidenceMatrix(indices, resolution, length(signal),4);

%Construct dual graph
W_d = abs(incidence_matrix * incidence_matrix');
W_d = W_d - diag(diag(W_d));

%Create graph structure
L_l = gsp_graph(W_d);  

%Compute Fourier basis
L_l = gsp_compute_fourier_basis(L_l);

%Compute edge weight and edge weight vector for discrete case 
[edge_weight,edge_signal_discrete] = ComputeEdgeWeight(signal,incidence_matrix);

%Compute optimal high bit-rate GFT
edge_signal_optimization = ComputeOptimizedWeightVector(signal, incidence_matrix, L_l);

%Computes GFT of signal f
edge_coefficients_discrete = gsp_gft(L_l,edge_signal_discrete);
edge_coefficients_optimization = gsp_gft(L_l,edge_signal_optimization);
edge_coefficients_optimization(1) = 0;

%sort data vector in descending order
[~,index_discrete] = sort(abs(edge_coefficients_discrete),'descend');    
[~,index_optimization] = sort(abs(edge_coefficients_optimization),'descend');

%Initialize new coefficient vectors
edge_coefficients_discrete_new = zeros(size(edge_coefficients_discrete));
edge_coefficients_optimization_new = zeros(size(edge_coefficients_optimization));

%Select largest coefficients
edge_coefficients_discrete_new(index_discrete(1)) = edge_coefficients_discrete(index_discrete(1)); 
edge_coefficients_optimization_new(index_optimization(1)) = edge_coefficients_optimization(index_optimization(1));
i = 2;

%Select remaining coefficients
while(i <= 5)
    if abs(edge_coefficients_optimization(index_optimization(i))) < 0.2*abs(edge_coefficients_optimization(index_optimization(1))) ...
            || abs(edge_coefficients_optimization(index_optimization(i))) < 0.4*abs(edge_coefficients_optimization(index_optimization(i-1)))
        graph_mode = 'continuous';
        break
    end
    
    edge_coefficients_discrete_new(index_discrete(i)) = edge_coefficients_discrete(index_discrete(i));
    edge_coefficients_optimization_new(index_optimization(i)) = edge_coefficients_optimization(index_optimization(i));
    
    i = i+1;
end

%Quantize coefficients
edge_coefficients_discrete_new = Delta*round(edge_coefficients_discrete_new/Delta);
index = find(edge_coefficients_optimization_new);
edge_coefficients_optimization_new(edge_coefficients_optimization_new<-2) = -2;
edge_coefficients_optimization_new(edge_coefficients_optimization_new>2) = 2;
edge_coefficients_optimization_new(index) = 0.5*(floor(edge_coefficients_optimization_new(index)/0.5)+0.5);


%Coefficients used
num_coef = nnz(edge_coefficients_optimization_new);

%Recreate edge weight vectors
edge_signal_discrete_new = gsp_igft(L_l,edge_coefficients_discrete_new);
edge_signal_optimization_new = gsp_igft(L_l,edge_coefficients_optimization_new);

%Rescales optimization weight signal
min_opt = min(edge_signal_optimization_new);
max_opt = max(edge_signal_optimization_new);
edge_signal_optimization_new = 0.95 * (edge_signal_optimization_new - min_opt)/(max_opt -min_opt) + 0.05;


if exist('graph_mode','var')
    edge_signal_new = edge_signal_optimization_new;
else
    %Discrete mode
    edge_signal_discrete_new = -sign(edge_signal_discrete_new);
    edge_signal_discrete_new(edge_signal_discrete_new == -1) = edge_weight;

    %Smoothness for each approach
    smoothness_discrete = signal' * incidence_matrix' * diag(edge_signal_discrete_new)*incidence_matrix * signal - sum(log(edge_signal_discrete_new));
    smoothness_optimization = signal' * incidence_matrix' * diag(edge_signal_optimization_new)*incidence_matrix * signal - sum(log(edge_signal_optimization_new));

    %Select mode based on smoothness
    if smoothness_discrete  < smoothness_optimization
        edge_signal_new = edge_signal_discrete_new; %discrete weights
    else
        edge_signal_new = edge_signal_optimization_new; %continuous weights
    end
end

%Recreate complex weight matrix
W = incidence_matrix' * diag(edge_signal_new)*incidence_matrix;
W = W - diag(diag(W));
end
        


