function [ W ] = CreateDominantWeightMatrix( indices, resolution, dominant_mode, signal)
%CreateDominantWeightMatrix Creates weight matrix of a dominant block
%
%   Input parameters:
%       indices         : Indices of image block in the image.
%       resolution      : Resolution of original image.
%       dominant_mode   : Group of quantized dominant gradient.
%       signal          : Signal on image block.
%   Ouput parameters:
%       W               : Quantized adapted weight matrix.

%Create incidence matrix
incidence_matrix = CreateIncidenceMatrix(indices, resolution, length(signal),8);

%Compute edge weight
[edge_weight,~] = ComputeEdgeWeight(signal,incidence_matrix);

%Initialize dominant weight matrix
W = zeros(length(signal));

res_horz = resolution(2);
res_vert = resolution(1);

%Create dominant weight matrix
for j = 1:size(indices,1)
    index = indices(j);

    %Check boundaries
    if index <= res_vert
        left_of_image = true;
    else
        left_of_image = false;
    end
    if index > res_vert*(res_horz-1)
        right_of_image = true;
    else
        right_of_image = false;
    end
    if mod(index,res_vert) == 1
        top_of_image = true;
    else
        top_of_image = false;
    end
    if mod(index,res_vert) == 0
        bottom_of_image = true;
    else
        bottom_of_image= false;
    end

    % pixel above
    if ~top_of_image && ismember(index-1,indices)
        if strcmp(dominant_mode,'vertical')
            W(j,j-1) = 1;
        elseif strcmp(dominant_mode,'ll_tr') || strcmp(dominant_mode,'lr_tl')
            W(j,j-1) = edge_weight;
        end
    end

    % pixel below
    if ~bottom_of_image && ismember(index+1,indices)
        if strcmp(dominant_mode,'vertical')
            W(j,j+1) = 1;
        elseif strcmp(dominant_mode,'ll_tr') || strcmp(dominant_mode,'lr_tl')
            W(j,j+1) = edge_weight;
        end
    end

    % pixel to the right
    if ~right_of_image && ismember(index+res_vert,indices)
        if strcmp(dominant_mode,'horizontal')
            W(j,indices == index+res_vert) = 1;
        elseif strcmp(dominant_mode,'ll_tr') || strcmp(dominant_mode,'lr_tl')
            W(j,indices == index+res_vert) = edge_weight;
        end
    end

    % pixel to the left
    if ~left_of_image && ismember(index-res_vert,indices)
        if strcmp(dominant_mode,'horizontal')
            W(j,indices == index-res_vert) = 1;
        elseif strcmp(dominant_mode,'ll_tr') || strcmp(dominant_mode,'lr_tl')
            W(j,indices == index-res_vert) = edge_weight;
        end
    end 

    % pixel top-left
    if ~top_of_image && ~left_of_image && ismember(index-res_vert-1,indices)
        if strcmp(dominant_mode,'lr_tl')
            W(j,indices == index-res_vert-1) = 1;
        elseif strcmp(dominant_mode,'horizontal') || strcmp(dominant_mode,'vertical')
            W(j,indices == index-res_vert-1) = edge_weight;
        end
    end

    % pixel top-right
    if ~top_of_image && ~right_of_image && ismember(index+res_vert-1,indices)
        if strcmp(dominant_mode,'ll_tr')
            W(j,indices == index+res_vert-1) = 1;
        elseif strcmp(dominant_mode,'horizontal') || strcmp(dominant_mode,'vertical')
            W(j,indices == index+res_vert-1) = edge_weight;   
        end
    end

    % pixel bottom-left
    if ~bottom_of_image && ~left_of_image && ismember(index-res_vert+1,indices)
        if strcmp(dominant_mode,'ll_tr')
            W(j,indices == index-res_vert+1) = 1;
        elseif strcmp(dominant_mode,'horizontal') || strcmp(dominant_mode,'vertical')
            W(j,indices == index-res_vert+1) = edge_weight;    
        end
    end

    % pixel bottom-right
    if ~bottom_of_image && ~right_of_image && ismember(index+res_vert+1,indices)
        if strcmp(dominant_mode,'lr_tl')
            W(j,indices == index+res_vert+1) = 1;
            elseif strcmp(dominant_mode,'horizontal') || strcmp(dominant_mode,'vertical')
            W(j,indices == index+res_vert+1) = edge_weight;    
        end
    end
end
end

